/*
 * AlMonitor.h
 *
 *      Author: Cesar_root
 */

#ifndef ALMONITOR_H_
#define ALMONITOR_H_

#include "type.h"

/*
 *
 *
 * Max number of rTx in case of fail(socket is not available)
 *
 */

#define mMAX_RETRY		1

/*
 *
 *
 *
 *
 * Signals available. Modes have to be declared after signal.
 *
 * Only Mode Indoor sends through WIFI
 *
 */

#define mFUNC_PANIC 1
#define mFUNC_ACCEL 2
#define mFUNC_ACACT 3
#define mFUNC_THERM 4
#define mGOTO_INDOOR 5
#define mGOTO_OUTDOOR 6
#define mGOTO_HYBRID 7

/*
 *
 *
 * Max number of functional alarms
 *
 */


#define mMAX_FUNC_AL 8

/*
 *
 *
 *
 *
 *
 *
 * Methods Prototypes
 *
 *
 *
 */


/** Creates queue to manage data to be sent
 *
 * @param void
 *
 * @return void
 */


void vAlmonitor_CreateQueue(void);


/** Get item(signal) from the queue
 *
 * @param xAlRx     structure of item(types.h)
 *
 * @return void
 */


void vAlMonitor_ReadAlarm(sSendAlarm *xAlRx);


/** Method Set. Add a signal to the queue ready to be sended
 *
 * @param xAlTx     structure of item(types.h)
 * @param bIsISR    true if method is called from a ISR
 *
 * @return void
 */


void vAlMonitor_AddAlarm(sSendAlarm *xAlTx,bool bIsISR);


/** Method Set. Set mode of operations.
 *
 * @param iNewMode  new mode to be set
 *
 * @return void
 */


void vAlMonitor_SetMode(int iNewMode);

/** Method Get. Get mode of operations.
 *
 * @param void
 *
 * @return iMode
 */


int8_t iAlMonitor_GetMode(void);


/** Send data to NCG. Choose Wifi or GSM
 *
 * @param sAlItem  Data structure to be sent
 * @param sToNCG   String code for the message.(indicates to NCG if its signal or data, format....)
 * @param iUID     Serial number ID of LPC1769
 *
 * @return true if it has been sent successfully
 */


bool vAlMonitor_ToNCG(sSendAlarm *sAlItem, char *sToNCG,unsigned *iUID[5]);


/** Method Get. Return number of items stored in queue ready to be sent.
 *
 * @param iNewMode  new mode to be set
 *
 * @return void
 */


int8_t  UAlMonitor_NumItems();


#endif /* ALMONITOR_H_ */


