/*
 * Thermometer.h
 *
 *  Created on: 25 de dic. de 2016
 *      Author: Cesar_root
 */

#ifndef THERMOMETER_H_
#define THERMOMETER_H_


/*
 *
 *
 *
 * Id for structures sended to monitor with data and no alarm.(First values is 100)
 *
 */

#define mDATA_THERM 100
/*
 *
 *
 *
 * Define the number of zones that are used(Range frome 2(1 LIMIT) to 7(6 LIMITS)
 *
 * */

#define mNUM_LIMITS  4

/*
 *
 *
 *
 *  Define the limits for temperature zones in ºC
 *
 * Usage:
 *
 *     mTEMPi    ºC
 *
 */

#define mTEMP1 	6
#define mTEMP2 	12
#define mTEMP3 	31
#define mTEMP4 	50
#define mTEMP5 	99				//Not used in this app
#define mTEMP6 	99	   			//Not used in this app

/*
 *
 *
 *
 * Define the resolution for monitoring of temperature changes.
 * If temperature changes more than mTEMP_RESOL, data and time stamp is sended to NCG
 *
 * Usage:
 *
 *     mTEMPi    ºC
 */

#define mTEMP_RESOL 3

/*
 *
 *
 *
 *
 *  Define time threshold to prevent false alarms. Value in seconds
 *
 * Usage:
 *
 *    mTEMP_TIME Seconds
 */

#define mTEMP_TIME	3


/*
 *
 *
 *
 *
 * Don`t change anything further than here
 *
 *
 *
 *
 */

#define mTEMP_LIM1	CELSIUS_TO_mV(mTEMP1)
#define mTEMP_LIM2	CELSIUS_TO_mV(mTEMP2)
#define mTEMP_LIM3	CELSIUS_TO_mV(mTEMP3)
#define mTEMP_LIM4	CELSIUS_TO_mV(mTEMP4)
#define mTEMP_LIM5	CELSIUS_TO_mV(mTEMP5)				//Not used in this app
#define mTEMP_LIM6	CELSIUS_TO_mV(mTEMP6)				//Not used in this app



/*
 * Time threshold
 */

#define mTEMP_THRES	 mTEMP_TIME*1000/portTICK_RATE_MS



/*
 *
 *
 *
 *
 *
 * Utils
 *
 *
 */

#define CELSIUS_TO_mV(cls) (((cls+54)*100/104))


/*
 *
 *
 *
 *
 *
 *
 * Methods Prototypes
 *
 *
 *
 */

/** Load temperature zone limits defined in header
 *
 * @param void
 *
 * @return void
 */

void Thermo_SetUp(void);

 /** Check if zone has changed after new reading of temperature
  *
  * @param iADC read
  *
  * @return bool true if zone has changed
  */

bool Thermo_ChkTemp(int16_t iADC);

/** Permanent monitor of temperature according to resolution defined in header(mTEMP_RESOL)
 *
 * @param iADC read
 *
 * @return bool true mTEMP_RESOL has been exceded
 */

bool Thermo_ChkTMonitor(int16_t iADC);

/** Return temperature from ADC read
 *
 * @param iADC value read from ADC
 *
 * @return temperature in ºC*100
 */

int16_t Thermo_GetTemp(int16_t iADC);

/** Get Method
 *
 * @param void
 *
 * @return Temperature zone
 */

int8_t Thermo_GetZone(void);

#endif /* THERMOMETER_H_ */
