/*****************************************************************************
 *   History
 *   2010.07.01  ver 1.01    Added support for UART3, tested on LPCXpresso 1700
 *   2009.05.27  ver 1.00    Prelimnary version, first Release
 *
******************************************************************************/
#ifndef __UART_H 
#define __UART_H

#include <stdint.h>


#define UART_PORT0 	0
#define UART_PORT1 	1
#define UART_PORT2	2
#define UART_PORT3 	3

#define IER_RBR		0x01
#define IER_THRE	0x02
#define IER_RLS		0x04

#define IIR_PEND	0x01
#define IIR_RLS		0x03
#define IIR_RDA		0x02
#define IIR_CTI		0x06
#define IIR_THRE	0x01

#define LSR_RDR		0x01
#define LSR_OE		0x02
#define LSR_PE		0x04
#define LSR_FE		0x08
#define LSR_BI		0x10
#define LSR_THRE	0x20
#define LSR_TEMT	0x40
#define LSR_RXFE	0x80


#define UART_RX_BUFFER 1000

/** Initialize UART port, setup pin select, clock, parity, stop bits, FIFO, etc.
 *
 * @param portNum UART port number (0,1,2,3).
 * @param baudRate UART port Baud rate.
 * @return
 */
uint32_t UART_init( uint32_t portNum, uint32_t baudRate );


void UART0_IRQHandler( void );
void UART1_IRQHandler( void );
void UART2_IRQHandler( void );
void UART3_IRQHandler( void );


/** Send a block of data to the UART 0 port based on the data length.
 *
 * @param portNum	UART port number (0,1,2,3).
 * @param BufferPtr	Buffer pointer.
 * @param Length	data length to send.
 */
void UART_Send( uint32_t portNum, uint8_t *BufferPtr, uint32_t Length );


#endif /* end __UART_H */
/*****************************************************************************
**                            End Of File
******************************************************************************/
