/*
 * wifly.h
 *
 *  Created on: Feb 18, 2016
 *      Author: jordi
 */

#ifndef WIFLY_H_
#define WIFLY_H_

#include "serial.h"



#define mWifiHomeSSID "cesar"
#define mWifiHomePSWD "3illo3illo"

/*
 *
 *
 * Time(ms) periode to update table of Wifis
 *
 */

#define mMILIS_UPDFW   4000

/*
 *
 *
 * Time(ms) periode for CommWatcher
 *
 */

#define mMILIS_COMM    30000



/*
 *
 *
 *  Select UART for Wifly
 *
 */

#define mWFPORT       (SERIAL_PORT1)

/*
 *
 *
 *  Select GPIO for reset pin of Wifly
 *
 */

#define WIFLY_RESET_PAD 6


/*
 *
 *
 *
 *
 *
 *
 * Methods Prototypes
 *
 *
 *
 */


/** Reset Wifly and call configure wifly function
 *
 * @param port      UART that Wifly is connected to
 * @param baudRate  Baudrate to communicate with Wifly through serial
 *
 * @return void
 */


void vWifly_init(uint8_t port, serial_baudrate baudRate);


/** Search home wifi and try to connect if it's detected
 *
 * @param void  whole string replied
 *
 * @return  true if done successfully
 */


bool Wifly_bWiFiSearch (void);


/** Check if net through wifly is up using ping command
 *
 * @param void
 *
 * @return  true  if it ping has reply
 */


bool Wifly_bWifiCheckPing(void);


/** Update table with MACs of three known wifi routers detected from home
 *
 * @param pPrevious  Last Tick that was called this function
 *
 * @return xTaskGetTickCount()
 */

portTickType Wifly_pUpdateFWTable(portTickType pPrevious);


/** Check if a known wifi signal is detected
 *
 * @param void
 *
 * @return  true if it's detected at least one
 */


bool Wifly_bCheckFWUp(void);



/** Open socket if it's needed and send STREAM DATA
 *
 * @param format,... Format and data to send
 *
 * @return true  if it is done successfully
 */

bool Wifly_bSendStream(int16_t iStream[],int16_t iNumElem,unsigned *iUID[5]);


/** Open socket if it's needed and send data
 *
 * @param format,... Format and data to send
 *
 * @return true  if it is done successfully
 */


bool Wifly_bSendMessageWifi(char* format, ...);


/** Get RTC time using sNTP client of Wifly
 *
 * @param void
 *
 * @return cTime RTC value
 */

char *Wifly_sGetTime();

#endif /* WIFLY_H_ */
