/*
 * AlMonitor.c
 *
 *
 *      Author: Cesar López
 *		Last Update: 03/01/2017
 */



#include "LPC17xx.h"
#include "log.h"
#include "FreeRTOS.h"
#include "config.h"
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "led.h"
#include "type.h"
#include "queue.h"
#include "log.h"
#include "AlMonitor.h"
#include "wifly.h"
#include <stdio.h>
#include "stdbool.h"


volatile xQueueHandle  xSendAlarm;
int8_t iMode=0;



/*
 * ----------------------------------------
 * ----------- LOCAL METHODS --------------
 * ----------------------------------------
 */


static void _vSendMessageGSM(sSendAlarm *sAlarm)
{
	Log_print(mGENER,"mCOMMU says: Sended alarm %i in state %i through GSM\r\n",sAlarm->iOPT,sAlarm->bIsON);
}


/*
 * ----------------------------------------
 * ----------- PUBLIC METHODS --------------
 * ----------------------------------------
 */


/** Method Set. Add a signal to the queue ready to be sended
 *
 * @param xAlTx     structure of item(types.h)
 * @param bIsISR    true if method is called from a ISR
 *
 * @return void
 */


void vAlMonitor_AddAlarm(sSendAlarm *xAlTx,bool bIsISR)
{
	if((xAlTx->iTry==0) && bIsISR==false)
	{
		xAlTx->cTime=(char *)Wifly_sGetTime();
	}
	xQueueSendToBack( xSendAlarm, xAlTx, 0 );
}


/** Get item(signal) from the queue
 *
 * @param xAlRx     structure of item(types.h)
 *
 * @return void
 */


void vAlMonitor_ReadAlarm(sSendAlarm *xAlRx)
{
	 xQueueReceive( xSendAlarm, xAlRx, portMAX_DELAY );
}


/** Method Set. Set mode of operations.
 *
 * @param iNewMode  new mode to be set
 *
 * @return void
 */


void vAlMonitor_SetMode(int iNewMode)
{
	iMode=iNewMode;
}

/** Method Get. Get mode of operations.
 *
 * @param void
 *
 * @return iMode
 */


int8_t iAlMonitor_GetMode(void)
{
	return iMode;
}

/** Method Get. Return number of items stored in queue ready to be sent.
 *
 * @param iNewMode  new mode to be set
 *
 * @return void
 */


int8_t  UAlMonitor_NumItems()
{
	return uxQueueMessagesWaiting(xSendAlarm);
}


/** Send data to NCG. Choose Wifi or GSM
 *
 * @param sAlItem  Data structure to be sent
 * @param sToNCG   String code for the message.(indicates to NCG if its signal or data, format....)
 * @param iUID     Serial number ID of LPC1769
 *
 * @return true if it has been sent successfully
 */


bool vAlMonitor_ToNCG(sSendAlarm *sAlItem, char *sToNCG,unsigned *iUID[5])
{


	if (iMode == mGOTO_INDOOR) //sAlItem->iValue,sAlItem->iParam,
	{
		if (Wifly_bSendMessageWifi("%s #%08X %08X %08X %08X #%i  #%i #%i #%i #%i #%i #%s \r\n",sToNCG, iUID[1],iUID[2],iUID[3],iUID[4],iMode,sAlItem->iOPT,sAlItem->bIsON,sAlItem->iTry,sAlItem->iValue,sAlItem->iParam,sAlItem->cTime)==false)
		{
			Log_print(mGENER,"mCOMMU says: Fail sending alarm in try %i Preparing to resend ->>> %s\r\n",sAlItem->iTry,sToNCG);
			sAlItem->iTry++;
			if (sAlItem->iTry > mMAX_RETRY)
			{
				_vSendMessageGSM(sAlItem);
			}
			else
			{
				vAlMonitor_AddAlarm(sAlItem,false);
			}
			return false;
		}
	}
	else if (iMode == mGOTO_HYBRID)
	{
		_vSendMessageGSM(sAlItem);
	}
	else
	{
		_vSendMessageGSM(sAlItem);
		return false;
	}
return true;
}


/** Creates queue to manage data to be sent
 *
 * @param void
 *
 * @return void
 */


void vAlmonitor_CreateQueue(void)
{
	xSendAlarm = xQueueCreate((int)mMAX_FUNC_AL, sizeof( sSendAlarm ) );
	if (xSendAlarm == NULL)
	{
	  Log_print(mGENER,"ERROR! Functional Alarm queue not created!\r\n");
	}
}


