/*
 * PanicButton.c
 *
 *
 *      Author: Cesar López
 *
 */


#include "LPC17xx.h"
#include "log.h"
#include "FreeRTOS.h"
#include "config.h"
#include "ButtonGPIO.h"
#include "type.h"
#include <string.h>
#include "AlMonitor.h"
#include "stdbool.h"
#include "gpio.h"
#include "utils.h"



/*
 * ----------------------------------------
 * ----------- LOCAL METHODS --------------
 * ----------------------------------------
 */


/** If SW_BOUNCE is true use delay to avoid bouncing. Otherwise condenser is assumed.
 *
 * @param void
 *
 * @return void
 */

void _ChkBouncing(void) {
	if (SW_BOUNCE==true) Utils_delayMs(500);
}


/*
 * ----------------------------------------
 * ----------- PUBLIC METHODS --------------
 * ----------------------------------------
 */


/** Override weak defined IRQ3 handler
 *
 * @param void
 *
 */

void EINT3_IRQHandler(void) {
	sSendAlarm xAlarmFuncRec;

	// If the rising edge interrupt was triggered1
	if((LPC_GPIOINT->IO0IntStatR >> 9) & 1)
	{
		LPC_GPIOINT->IO0IntClr |= (1 << 9);
		_ChkBouncing();

		xAlarmFuncRec.iOPT=mFUNC_PANIC;
		xAlarmFuncRec.iTry=0;
		vAlMonitor_AddAlarm(&xAlarmFuncRec,true);
	}
return;
}

/** Init the PanicButton, set GPIO, enable Interrupt
 *
 */
void ButtonGPIO_vInit(void)
{

	// Configure Pin P0[9] for Panic Button
	Gpio_config(PINBUTTON, GPIO_IN );
		
	// Enable Rising Edge Interrupt on P0[9]
	LPC_GPIOINT->IO0IntEnR |= (1 << 9);
	
	NVIC_EnableIRQ(EINT3_IRQn);

	return;
	
}


