/*
 * UartScanf.c
 *
 *  Created on: 31 de dic. de 2016
 *      Author: Cesar_root
 */

#include "utils.h"
#include "serial.h"
#include <stdint.h>
#include "LPC17xx.h"
#include <string.h>
#include "log.h"
#include "config.h"
#include "FreeRTOS.h"
#include "uart.h"
#include "wifly.h"
#include "semphr.h"

extern volatile uint32_t UART3Count;
extern volatile uint8_t UART3Buffer[UART_RX_BUFFER];

extern volatile uint32_t UART1Count;
extern volatile uint8_t UART1Buffer[UART_RX_BUFFER];

volatile static xSemaphoreHandle ScanLogMutex = NULL;

/** Manage semaphore while scanning UART_LOG
 *
 * @param void
 *
 * @return void
 */

static void _vLockScanLog()
{

	if(ScanLogMutex == NULL)
	{
		ScanLogMutex = xSemaphoreCreateMutex();
	}

	xSemaphoreTake(ScanLogMutex, portMAX_DELAY);
}

/** Manage semaphore while scanning UART_LOG
 *
 * @param void
 *
 * @return void
 */

static void _vUnlockScanLog()
{
	xSemaphoreGive(ScanLogMutex);
}

/** Read characters sended by device to LPC1769 through any UART
 *
 * @param iPort       Uart to be read
 * @param iTimeOut    Time to wait listening
 * @param sResponse	  data read
 *
 * @return 0 in case of error, otherwise returns the size of the buffer
 */

uint_fast8_t UartScanf_iScanSerial(uint_fast8_t iPort,uint32_t iTimeOut,char *sResponse)
{
	uint32_t indice = 0;
	uint32_t indice1 = 0;

	switch(iPort)
			{
				case mUART_LOG:
					_vLockScanLog();
					UART3Count = 0; // clean buff
					LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR;
					Utils_delayMs(iTimeOut);
					LPC_UART3->IER = IER_THRE | IER_RLS;
					if (UART3Count!=0)
					{
						strncpy(sResponse, (char *)UART3Buffer, 1 );  //Only first char is usefull
					}

					_vUnlockScanLog();
					return UART3Count;
					break;

				case mWFPORT:
					UART1Count = 0; /* Prepare to start receiving */
					/* Enable Serial NVIC interrupts and wait some time */
					LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR;/* Re-enable RBR */
					Utils_delayMs(iTimeOut); /* Wait for 2 seconds */
					/* Let's wait for all the chars to arrive */
					do{
						indice = indice1;
						Utils_delayMs(10);
						indice1 = UART1Count;
					}while(indice != indice1);
					LPC_UART1->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
					strncpy(sResponse,(char *)UART1Buffer,indice1); sResponse[indice1]='\0'; // End string
					break;
			}
return indice1;
}

/** Clear Uart1 or Uart3
 *
 * @param iPort   Uart to be cleared
 *
 * @return void
 */

void UartScanf_UARTClear(uint_fast8_t iPort)
{
	switch(iPort)
	{
	case mUART_LOG:
		UART3Count = 0;
		UART3Buffer[0]='\0';
		break;
	case mWFPORT:
		UART1Count = 0;
		UART1Buffer[0]='\0';
		break;
	}
}
