/*
 * serial.c
 *
 *  Created on: Feb 4, 2016
 *      Author: jordi
 */

#include "assertlib.h"

#include <stdint.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "led.h"
#include "type.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "uart.h"
#include "serial.h"
#include "utils.h"



// @see serial_baudrates (use same order for vector)

char serialTxBuffer[SERIAL_MAX_PORT][SERIAL_TX_BUFFER];
xSemaphoreHandle mutex_serial[SERIAL_MAX_PORT] = {NULL};


/* Initialize serial port
 *
 * @param port Serial port to use
 * @param baudrate to use
 *
 * @return TRUE if all ok, othewise FALSE
 */


uint_fast8_t Serial_start( uint_fast8_t port, serial_baudrate baudRate ) {

	xSemaphoreHandle *pMux;
	pMux = &(mutex_serial[port]);

	if ((port >= SERIAL_MAX_PORT )
		|| (baudRate == BAUD_UNSUPORTED) ){
		assert(0);
		return FALSE;
	}

	if (*pMux == NULL) *pMux = xSemaphoreCreateRecursiveMutex();
	xSemaphoreTake( *pMux, portMAX_DELAY );
	UART_init(port, baudRate);
	xSemaphoreGive( *pMux );

	return TRUE;
}


/*
 * Send data via serial port
 *
 * @param port 		uart port to send data
 * @param format 	like printf of cstdio C standard
 */


uint_fast16_t Serial_vsend(uint_fast8_t port, char *format, va_list args) {
	xSemaphoreHandle *pMux;
	uint16_t lenBuff = 0;



	pMux = &(mutex_serial[port]);


	// check valid port
	if (port >= SERIAL_MAX_PORT ) {
		assert(0);
		return FALSE;
	}

	// serial has to be initialized before
	if (*pMux == NULL) {
		assert(0);
		return FALSE;
	}


	xSemaphoreTake( *pMux, portMAX_DELAY );
	{
		char* pBuffSerialTx;

		pBuffSerialTx = &(serialTxBuffer[port][0]);

		lenBuff = vsprintf(pBuffSerialTx, format, args);

		UART_Send(port, (uint8_t*)pBuffSerialTx, lenBuff);

	}
	xSemaphoreGive( *pMux );

	return lenBuff;
}


/*
 *
 * @param port 		uart port to send data
 * @param format  	string that contains a format string that follows the same specifications as format in Serial_send
 */

uint_fast16_t Serial_send(uint_fast8_t port, char *format, ...) {

	uint_fast16_t len;
	va_list args;

	va_start( args, format );
	len = Serial_vsend(port, format, args);
	va_end(args);

	return len;
}


