/*
 * wifly.c
 *
 *  Created on: Feb 18, 2016
 *      Author: jordi
 */

#include "assertlib.h"

#include "type.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "stdbool.h"
#include "config.h"
#include "task.h"
#include "led.h"
#include "Almonitor.h"
#include "log.h"
#include "string.h"
#include "wifly.h"
#include "UartScanf.h"
#include "serial.h"
#include "gpio.h"
#include "utils.h"


static int8_t iIsSocket = 0;

xSemaphoreHandle wiflyMux;
static char cWiflyScanf[UART_RX_BUFFER];
static char sFriendlyWifi[3][18];
static char cTime[30];


/*
 * ----------------------------------------
 * ----------- LOCAL METHODS --------------
 * ----------------------------------------
 */


static bool _bWfCheckCmdResult(uint32_t iTime);
static bool _bWifiGoCmd(void);
static bool _bWifiExitCmd(void);
static bool _bWifiConfigJoin(void);
static void _sParseScan(char *sScanFW);
static void _vParseRTC(char *sFindRTC);


/** Reset Wifly and call configure wifly function
 *
 * @param void
 *
 * @return void
 */

static void _resetWifly() {
	Gpio_config(WIFLY_RESET_PAD, GPIO_OUT);
	Gpio_set(WIFLY_RESET_PAD, GPIO_HIGH);
	Utils_delayMs(1000);	// min 160us
	Gpio_set(WIFLY_RESET_PAD, GPIO_LOW);
	Utils_delayMs(1000);	// min 160us
	Gpio_set(WIFLY_RESET_PAD, GPIO_HIGH);

	Utils_delayMs(1000);	// min 160us
	_bWifiConfigJoin();

}

/** Set Wifly in command mode
 *
 * @param void
 *
 * @return true  true if it's detected at least one
 */

static bool _bWifiGoCmd(void)
{
	memset(cWiflyScanf, 0, UART_RX_BUFFER);
	UartScanf_UARTClear(mWFPORT);
	Log_print(mCOMMU,"mCOMMU says: Entering command mode (Wifly)\r\n");
	Serial_send(mWFPORT,"$$$");
	return _bWfCheckCmdResult(200);
}


/** Configure Wifly, save and reboot
 *
 * @param void
 *
 * @return true  if it is done successfully
 */

static bool _bWifiConfigJoin(void)
{
	if (_bWifiGoCmd()==false) return false;

	Serial_send(mWFPORT,"set wlan auth 0\r\n");
	if (_bWfCheckCmdResult(300)==false) return false;

	Serial_send(mWFPORT,"set ip dchp 1\r\n");
	if (_bWfCheckCmdResult(300)==false)  return false;

	Serial_send(mWFPORT,"set wlan phrase 3illo3illo\r\n");
	if (_bWfCheckCmdResult(300)==false)  return false;
/*
	Serial_send(mWFPORT,"set wlan phrase 3illo3illo\r\n");
	if (_bWfCheckCmdResult(300)==false)  return false;
*/
	Serial_send(mWFPORT,"set ip proto 18\r\n");
	if (_bWfCheckCmdResult(300)==false)  return false;

	Serial_send(mWFPORT,"set ip remote 10000\r\n");
	if (_bWfCheckCmdResult(300)==false)  return false;

	Serial_send(mWFPORT,"set dns name arpalab.appspot.com\r\n");
	if (_bWfCheckCmdResult(300)==false)  return false;

	Serial_send(mWFPORT,"set option format 0\r\n");
	if (_bWfCheckCmdResult(300)==false)  return false;

	Serial_send(mWFPORT,"set com remote 0\r\n");
	if (_bWfCheckCmdResult(300)==false)  return false;

	Serial_send(mWFPORT,"set time address 130.206.3.166\r\n");
	if (_bWfCheckCmdResult(300)==false)  return false;

	Serial_send(mWFPORT,"set time z 23\r\n");
	if (_bWfCheckCmdResult(300)==false)  return false;

	Serial_send(mWFPORT,"save\r\n");
	if (_bWfCheckCmdResult(300)==false)  return false;
	if (_bWifiExitCmd()==false) return false;
	return true;
}

/** Join to WIFI LAN
 *
 * @param void
 *
 * @return true  if it is done successfully
 */

static bool _bWifiSetJoin(void)
{
	Serial_send(mWFPORT,"join cesar\r\n");
	if (_bWfCheckCmdResult(1500)==false) return false;

	// Launch NTP client

	Serial_send(mWFPORT,"set time enable 1\r\n");
	if (_bWfCheckCmdResult(300)==false)  return false;
	return true;
}


/** Exit command mode and turn Wifly in data mode
 *
 * @param void
 *
 * @return  true  if it is done successfully
 */

static bool _bWifiExitCmd(void)
{
	memset(cWiflyScanf, 0, UART_RX_BUFFER);
	UartScanf_UARTClear(mWFPORT);
	Log_print(mCOMMU,"mCOMMU says: Exiting command mode (Wifly) %d\r\n",xTaskGetTickCount());
    Serial_send(mWFPORT,"exit\r\n");
    return _bWfCheckCmdResult(200);

}

/** Check if any command sent to Wifly went well or wrong listening the reply.
 *
 * @param iTime  Time to wait listening
 *
 * @return  true  if command went well
 */


static bool _bWfCheckCmdResult(uint32_t iTime)
{
	uint_fast8_t iNumCh;

	memset(cWiflyScanf, 0, UART_RX_BUFFER);
	UartScanf_UARTClear(mWFPORT);

	iNumCh=UartScanf_iScanSerial(mWFPORT,iTime,cWiflyScanf);

	if (iNumCh!=0)
	{
		if (iTime<301)
		{
			if ((strstr(cWiflyScanf, "CMD") != NULL)  || (strstr(cWiflyScanf, "AOK") != NULL)|| (strstr(cWiflyScanf, "OK") != NULL) || (strstr(cWiflyScanf, "EXIT") != NULL)|| (strstr(cWiflyScanf, "reply") != NULL))
			{
				Log_print(mCOMMU,"mCOMMU says: Command went WELL!! Result is %i bytes in: %s\r\n",iNumCh,cWiflyScanf);
				return true;
			}
		}
		else if (iTime==301)
		{
			_vParseRTC(cWiflyScanf);
			return true;
		}
		else if (iTime==601)
		{
			if ((strstr(cWiflyScanf, "*OPEN") != NULL) || ((strstr(cWiflyScanf, "Connected"))))
			{
				Log_print(mCOMMU,"OPEN command went well\r\n");
				return true;
			}
			else if(strstr(cWiflyScanf, "*CLOS*") != NULL)
			{
				Log_print(mCOMMU,"mCOMMU says: RECEIVED CLOSE SIGNAL is %i bytes in: %s\r\n",iNumCh,cWiflyScanf);
				return true;
			}
			else
			{
				Log_print(mCOMMU,"mCOMMU says: OPEN SOCKET HAS FAILED is %i bytes in: %s\r\n",iNumCh,cWiflyScanf);
				return false;
			}
		}
		else if (iTime==1500)
		{
			if (strstr(cWiflyScanf, "Associated!") != NULL)
			{
				Log_print(mCOMMU,"mCOMMU says: Command JOIN went WELL!! Result is %i bytes in: %s\r\n",iNumCh,cWiflyScanf);
				return true;
			}
			else
			{
				Log_print(mCOMMU,"mCOMMU says: Command JOIN went WRONG CLOSING!! Result is %i bytes in: %s\r\n",iNumCh,cWiflyScanf);
			}
		}
		else if (iTime == 2002)
		{
			//It is a scan command for updating FW table
			_sParseScan(cWiflyScanf);
			return true;
		}
		else if (iTime == 2001)
		{
			//It is a scan command for checking FW
			if (strlen(sFriendlyWifi[0])>1)
			{
				if ((strstr(cWiflyScanf,sFriendlyWifi[0] ) != NULL) || (strstr(cWiflyScanf, sFriendlyWifi[1]) != NULL)|| (strstr(cWiflyScanf, sFriendlyWifi[2]) != NULL))
				{
					Log_print(mCOMMU,"mCOMMU says: Checking FW table(at least one match founded, Hybrid mode\r\n");
					return true;
				}
			}
		}
	}
	_bWifiExitCmd();
	return false;
}

/** Get RTC value from show time t command reply.
 *
 * @param sFindRTC  whole string replied
 *
 * @return  void
 */

static void _vParseRTC(char *sFindRTC)
{
	char *sTokenLine;

	for (sTokenLine = strtok (sFindRTC, "\r\n"); sTokenLine != NULL;	sTokenLine = strtok (sTokenLine + strlen (sTokenLine) + 1, "\n"))
	{
		if (strstr(sTokenLine,"RTC")!=0)
		{
			strncpy (cTime, strstr(sTokenLine,"=")+1, strlen (sTokenLine)-3);
			break;
		}
	}
return;
}

/** Get MAC addresses of three Wifi routers detected from home
 *
 * @param sScanFW  whole string replied
 *
 * @return  void
 */

static void _sParseScan(char *sScanFW)
{
	char *sTokenLine;
	char *sTokenWord;
	char buf[256];
	int iCount = 0;
    Log_print(mCOMMU,"mCOMMU says: Received scan list %s\r\n",sScanFW);
    Log_print(mCOMMU,"mCOMMU says: List of FW stored \r\n%s\r\n%s\r\n%s\r\n",sFriendlyWifi[0],sFriendlyWifi[1],sFriendlyWifi[2]);
	for (sTokenLine = strtok (sScanFW, "\r\n"); sTokenLine != NULL;	sTokenLine = strtok (sTokenLine + strlen (sTokenLine) + 1, "\n"))
	{
		strncpy (buf, sTokenLine, sizeof (buf));
		for (sTokenWord = strtok (buf, ","); sTokenWord != NULL;sTokenWord = strtok (sTokenWord + strlen (sTokenWord) + 1, ","))
		{
			if ((strlen(sTokenWord)==17))
			{
				if (strcmp(sTokenWord,sFriendlyWifi[iCount])!=0)
				{
					if (strcmp(sTokenWord,"b4:9d:0b:31:05:66")!=0)
					{
						strncpy(sFriendlyWifi[iCount],sTokenWord,17);
						strcat(sFriendlyWifi[iCount],'\0');
						Log_print(mCOMMU,"mCOMMU says: Updated FW table, added new router MAC = %s in POSITION %i\r\n",sFriendlyWifi[iCount],iCount);
						iCount++;
					}
				}
				else
				{
					iCount++;
				}
			}
		}
		if (iCount == 3) break;
	}
	return;
}

/*
 * ----------------------------------------
 * ----------- PUBLIC METHODS --------------
 * ----------------------------------------
 */

/** Reset Wifly and call configure wifly function
 *
 * @param port      UART that Wifly is connected to
 * @param baudRate  Baudrate to communicate with Wifly through serial
 *
 * @return void
 */


void vWifly_init(uint8_t port, serial_baudrate baudRate) {
    Log_print(mGENER,"mCOMMU says: Initializing Wifly module!\r\n");

	if (wiflyMux == NULL) wiflyMux = xSemaphoreCreateRecursiveMutex();
	xSemaphoreTake( wiflyMux, portMAX_DELAY );
	{
		if (!Serial_start(port, baudRate)){
			assert(0;)
		}

		_resetWifly();
	}
	xSemaphoreGive( wiflyMux );

}

/** Update table with MACs of three known wifi routers detected from home
 *
 * @param pPrevious  Last Tick that was called this function
 *
 * @return xTaskGetTickCount()
 */

portTickType Wifly_pUpdateFWTable(portTickType pPrevious)
{
	xSemaphoreTake( wiflyMux, portMAX_DELAY );

	if ((xTaskGetTickCount()-pPrevious) < mMILIS_UPDFW/portTICK_RATE_MS)
	{
		xSemaphoreGive( wiflyMux);
		return pPrevious;
	}

	_bWifiGoCmd();
	Serial_send(mWFPORT,"scan\r\n");
	_bWfCheckCmdResult(2002);
	_bWifiExitCmd();
	xSemaphoreGive( wiflyMux);

	return xTaskGetTickCount();
}

/** Check if a known wifi signal is detected
 *
 * @param void
 *
 * @return true if done successfully
 */

bool Wifly_bCheckFWUp(void)
{
	xSemaphoreTake( wiflyMux, portMAX_DELAY );

	if (_bWifiGoCmd()==false)
	{
		xSemaphoreGive( wiflyMux);
		return false;
	}
	Serial_send(mWFPORT,"scan\r\n");

	if (_bWfCheckCmdResult(2001)==false)
	{
		xSemaphoreGive( wiflyMux);
		return false;
	}
	if (_bWifiExitCmd()==false)
	{
		xSemaphoreGive( wiflyMux);
		return false;
	}
	xSemaphoreGive( wiflyMux);
	return true;
}


/** Open socket if it's needed and send STREAM DATA
 *
 * @param format,... Format and data to send
 *
 * @return true  if it is done successfully
 */

bool Wifly_bSendStream(int16_t iStream[],int16_t iNumElem,unsigned *iUID[5])
{
	xSemaphoreTake( wiflyMux, portMAX_DELAY );
	if (iIsSocket==0)
	{
	if (_bWifiGoCmd()==false)
	{
		xSemaphoreGive( wiflyMux );
		return false;
	}

	Serial_send(mWFPORT,"open 192.168.43.180 10000\r\n");
	if (_bWfCheckCmdResult(601)==false)
	{
		_bWifiExitCmd();
		xSemaphoreGive( wiflyMux );

		return false;
	}
	else
	{
		iIsSocket=1;
	}
}

	Serial_send(mWFPORT,"DMP_STREAM # %08X %08X %08X %08X",iUID[1],iUID[2],iUID[3],iUID[4]);


	for ( int i=0 ; i<iNumElem ; i++ ) 	Serial_send(mWFPORT,"#%d",iStream[i]);

	Serial_send(mWFPORT,"\r\n");
	Serial_send(mWFPORT,"quit\r\n");
	Utils_delayMs(500);
	iIsSocket=0;


	UartScanf_UARTClear(mWFPORT);

	xSemaphoreGive( wiflyMux );
	return true;

}

/** Open socket if it's needed and send data
 *
 * @param format,... Format and data to send
 *
 * @return true  if it is done successfully
 */

bool Wifly_bSendMessageWifi(char* format, ...)
{

	xSemaphoreTake( wiflyMux, portMAX_DELAY );
	if (iIsSocket==0)
	{
	if (_bWifiGoCmd()==false)
	{
		xSemaphoreGive( wiflyMux );
		return false;
	}

	Serial_send(mWFPORT,"open 192.168.43.180 10000\r\n");
	if (_bWfCheckCmdResult(601)==false)
	{
		_bWifiExitCmd();
		xSemaphoreGive( wiflyMux );

		return false;
	}
	else
	{
		iIsSocket=1;
	}
}
	va_list args;
	va_start( args, format );
	Serial_vsend(mWFPORT, format, args);
	va_end(args);
	if (UAlMonitor_NumItems()==0)
	{
		Serial_send(mWFPORT,"quit\r\n");
		Utils_delayMs(500);
		iIsSocket=0;
	}

	UartScanf_UARTClear(mWFPORT);

	xSemaphoreGive( wiflyMux );
	return true;

}

/** Get RTC time using sNTP client of Wifly
 *
 * @param void
 *
 * @return cTime RTC value
 */

char *Wifly_sGetTime()
{
	xSemaphoreTake( wiflyMux, portMAX_DELAY );

	if (_bWifiGoCmd()==false)
	{
		xSemaphoreGive( wiflyMux );
		return false;
	}

	Serial_send(mWFPORT,"show time t\r\n");

	_bWfCheckCmdResult(301);

	_bWifiExitCmd();
	xSemaphoreGive(wiflyMux);

	Log_print(mCOMMU,"mCOMMU says: Time taked from NTP server %s\r\n",cTime);

	return cTime;
}

/** Check if net through wifly is up using ping command
 *
 * @param void
 *
 * @return  true  if it ping has reply
 */

bool Wifly_bWifiCheckPing(void)
{
	xSemaphoreTake( wiflyMux, portMAX_DELAY );

	if (_bWifiGoCmd()==false)
	{
		xSemaphoreGive( wiflyMux);
		return false;
	}
	Serial_send(mWFPORT,"ping google.es\r\n");
	if (_bWfCheckCmdResult(300)==false)
	{
		xSemaphoreGive( wiflyMux);
		return false;
	}
	_bWifiExitCmd();
	xSemaphoreGive( wiflyMux);
	return true;
}


/** Search home wifi and try to connect if it's detected
 *
 * @param void  whole string replied
 *
 * @return  true if done successfully
 */

bool Wifly_bWiFiSearch ()
{
	xSemaphoreTake( wiflyMux, portMAX_DELAY );

	if (_bWifiGoCmd()==false)
	{
		xSemaphoreGive( wiflyMux );
		return false;
	}

	if (_bWifiSetJoin()==false)
	{
		xSemaphoreGive( wiflyMux );
		return false;
	}

	if (_bWifiExitCmd()==false)
	{
		xSemaphoreGive( wiflyMux );
		return false;
	}

	xSemaphoreGive( wiflyMux );
	return true;
}



