/*
 *
 *      Author: Cesar López
 *
 */

/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "LPC17xx.h"
#include "type.h"
#include "led.h"
#include "serial.h"
#include "log.h"
#include "adc.h"
#include "Pulse.h"
#include "config.h"
#include "string.h"
#include "ButtonGPIO.h"
#include "Accelerometer.h"
#include "Thermometer.h"
#include "AlMonitor.h"
#include "utils.h"
#include <stdlib.h>
#include "wifly.h"




// Home Wifi SSID and PSW
char sWifiHomeSSID[] = "cesar";
char sWifiHomePWSD[] = "3illo3illo";
unsigned *uUID[5];
int8_t UAlarmUp=0;
sSendAlarm aAlarmFuncRec[mMAX_FUNC_AL];


static void vLogTask( void *pvParameters );
static void vAlarmMonitor( void *pvParameters );
static void vCommWatcher( void *pvParameters );
static void vADCSensors( void *pvParameters );


//Auxiliary functions
void vGeneralInit(void);
bool bNoWifi(void);
void vWorkAlarmUp(sSendAlarm sSAlarm[],int iPos,sSendAlarm *sToNCG);
void vWorkAlarmDown(sSendAlarm sSAlarm[],int iPos,sSendAlarm *sTNCG);


int main(void)
{

	 portBASE_TYPE xReturned;


	//Init vars, queues, etc...

	vGeneralInit();


	xReturned=xTaskCreate( vLogTask,( signed char * ) "LogTask", configTYP_STACK_SIZE, NULL, TaskNormMode, NULL );

	if (xReturned != pdPASS ) Log_print(mGENER,"Log Task Task COULD NOT BE CREATED\r\n");
	else Log_print(mGENER,"Log Task created successfully\r\n");

	xReturned=xTaskCreate( vADCSensors,( signed char * ) "ADCSensors", configTYP_STACK_SIZE, NULL, TaskNormMode, NULL );

	if (xReturned != pdPASS ) Log_print(mGENER,"ADCSensors Task COULD NOT BE CREATED\r\n");
	else Log_print(mGENER,"ADCSensors Task created successfully\r\n");

	xReturned=xTaskCreate( vCommWatcher,( signed char * ) "CommWatcher", configTYP_STACK_SIZE, NULL, TaskNormMode, NULL );

	if (xReturned != pdPASS ) Log_print(mGENER,"CommWatcher Task COULD NOT BE CREATED\r\n");
	else Log_print(mGENER,"CommWatcher Task created successfully\r\n");

	xReturned=xTaskCreate( vAlarmMonitor,( signed char * ) "AlarmMonitor", configTYP_STACK_SIZE, NULL, TaskNormMode, NULL );

	if (xReturned != pdPASS ) Log_print(mGENER,"AlMonitor Task COULD NOT BE CREATED\r\n");
	else Log_print(mGENER,"AlMonitor Task created successfully\r\n");


	vTaskStartScheduler();

	for( ;; );
}

/** Create task that handles the link between DMP and NCG
 *
 * @void *pvParameters
 */

void vStateToMonitor(sSendAlarm *xNewState, int8_t iState)
{
	xNewState->bIsON=true;
	xNewState->iTry=0;
	if (iState == mGOTO_INDOOR)
	{
		xNewState->iOPT=mGOTO_INDOOR;
		vAlMonitor_AddAlarm(xNewState,false);

	}
	else if (iState == mGOTO_OUTDOOR)
	{
		if (xNewState->iOPT!=mGOTO_OUTDOOR)
		{
			xNewState->iOPT=mGOTO_OUTDOOR;
			vAlMonitor_AddAlarm(xNewState,false);
		}
	}
	else if (iState == mGOTO_HYBRID)
	{
		if (xNewState->iOPT!=mGOTO_HYBRID)
		{
			xNewState->iOPT=mGOTO_HYBRID;
			vAlMonitor_AddAlarm(xNewState,false);

		}
	}
}

static void vCommWatcher( void *pvParameters )
{
 bool bIsWifi = false;
 sSendAlarm xAlarmChangeState;
 xAlarmChangeState.iOPT=0;
 xAlarmChangeState.bIsON=false;
 portTickType pxPrevUpdate=-1;

 pxPrevUpdate=xTaskGetTickCount();
	for( ;; )
	{
		if (bIsWifi == true)
		{
			 //End of trying
			//Check Connection doing Ping
			if (Wifly_bWifiCheckPing()==false)
			{
				 bIsWifi = bNoWifi();
				 if (bIsWifi==false)
				 {
					 //if bIsWifi is false Wifi lost, Ping and retrying to connect have failed,
					 //send ChangeToHybrid o Outdoor to Monitor
						if(Wifly_bCheckFWUp()==true)
						{
							vStateToMonitor(&xAlarmChangeState,mGOTO_HYBRID);
						}
						else
						{
							vStateToMonitor(&xAlarmChangeState,mGOTO_OUTDOOR);
						}
				 }
				 else
				 {
					 //if connection recovered update Friendly Wifis Table
					 pxPrevUpdate = Wifly_pUpdateFWTable(pxPrevUpdate);
				 }
			}
			else
			{
				//connection wifi is OK so update Friendly Wifis Table do not send message
				pxPrevUpdate = Wifly_pUpdateFWTable(pxPrevUpdate);
			}
		}
		else
		{
			bIsWifi = bNoWifi();
			if(bIsWifi == true)
			{
				while(Wifly_bWifiCheckPing()==false);
				vStateToMonitor(&xAlarmChangeState,mGOTO_INDOOR);
				pxPrevUpdate = Wifly_pUpdateFWTable(pxPrevUpdate);
			}
			else
			{
				if(Wifly_bCheckFWUp()==true)
				{
					vStateToMonitor(&xAlarmChangeState,mGOTO_HYBRID);
				}
				else
				{
					vStateToMonitor(&xAlarmChangeState,mGOTO_OUTDOOR);
				}
			}
		}
		vTaskDelay( MILIS_TO_TICKS(mMILIS_COMM));
	}
}

bool bNoWifi(void)
{

	if (Wifly_bWiFiSearch() == true)
	{
		Log_print(mCOMMU,"mCOMMU says: Founded Home WIFI\r\n");
		return true;
	}
	else
	{
		Log_print(mCOMMU,"mCOMMU says: Not founded Home WIFI\r\n");

	}
return false;
}

/*-----------------------------------------------------------*/

/** Create task that watches the data provided by sensors and take decisions
 *
 * @void *pvParameters
 */

static void vAlarmMonitor( void *pvParameters )
{


	sSendAlarm xAlarmFuncRec;
	//Just in case it is the Panic Button, then it has to toggle;
    bool bON=false;

    //Init PanicButton alarm to false
    aAlarmFuncRec[mFUNC_PANIC].bIsON=false;

	for( ;; )
	{
		vAlMonitor_ReadAlarm(&xAlarmFuncRec);

		if (xAlarmFuncRec.iOPT>99)
		{
			Log_print(mGENER,"APP says: Sended DATA from %d to NCG value is %d\r\n",xAlarmFuncRec.iOPT,xAlarmFuncRec.iValue);

			vAlMonitor_ToNCG(&xAlarmFuncRec,(char *)"DMP_DATA",uUID);
		}
		else
		{
			if (xAlarmFuncRec.iTry==0)
			{
				bON=aAlarmFuncRec[xAlarmFuncRec.iOPT].bIsON;
				aAlarmFuncRec[xAlarmFuncRec.iOPT] = xAlarmFuncRec;
				//If is the panic button and first tx, alarm must be toggled.
				if (xAlarmFuncRec.iOPT==mFUNC_PANIC)
				{
					aAlarmFuncRec[xAlarmFuncRec.iOPT].bIsON = !(bON);
					xAlarmFuncRec.bIsON = aAlarmFuncRec[xAlarmFuncRec.iOPT].bIsON ;
				}
				//End of checking
			}
			if (aAlarmFuncRec[xAlarmFuncRec.iOPT].bIsON==true)
			{
				//Alarm has been activated
				vWorkAlarmUp(aAlarmFuncRec,xAlarmFuncRec.iOPT,&xAlarmFuncRec);
			}
			else
			{
				//Alarm has been deactivated
				vWorkAlarmDown(aAlarmFuncRec,xAlarmFuncRec.iOPT,&xAlarmFuncRec);
			}
		}
	}
}

/** Do what it takes if Alarm Monitor  noticed an alarm going UP
 *
 *  @param sSendAlarm sSAlarm[]: Array with all alarms data(maybe there are consequences for another alarm)
 *  @param int iPos: position in the array of the current alarm
 *
 */


void vWorkAlarmUp(sSendAlarm sSAlarm[],int iPos,sSendAlarm *sToNCG)
{

	if (sToNCG->iTry==0)
	{

		switch(sSAlarm[iPos].iOPT)
		{
		case mFUNC_PANIC:
		{
			UAlarmUp++;
			sToNCG->cTime=(char *)Wifly_sGetTime();
			Log_print(mGENER,"APP says: Received Alarm from PanicButton and it is %i\r\n",sSAlarm[iPos].iOPT,sSAlarm[iPos].bIsON);
			break;
		}
		case mFUNC_ACCEL:
		{
			UAlarmUp++;
			Log_print(mGENER,"APP says: Received Alarm from mACCEL FALL and it is %i\r\n",sSAlarm[iPos].bIsON);
			break;
		}
		case mFUNC_ACACT:
		{
			UAlarmUp++;
			Log_print(mGENER,"APP says: Received Alarm from mACCEL ACTIVITY and it is %i\r\n",sSAlarm[iPos].bIsON);
			break;
			}
		case mFUNC_THERM:
		{
			Log_print(mGENER,"APP says: Received Alarm from THERM, zone changed to: %i and temp is: %i\r\n",sSAlarm[iPos].iParam,sSAlarm[iPos].iValue);
			break;
			}
		case mGOTO_INDOOR:
		{
			Log_print(mGENER,"APP says: STATE HAS CHANGE TO INDOOR \r\n");
			vAlMonitor_SetMode(mGOTO_INDOOR);
			break;
		}
		case mGOTO_HYBRID:
		{
				vAlMonitor_SetMode(mGOTO_HYBRID);
				Log_print(mGENER,"APP says:  STATE HAS CHANGE TO HYBRID\r\n");
			break;
		}
		case mGOTO_OUTDOOR:
		{
				vAlMonitor_SetMode(mGOTO_OUTDOOR);
				Log_print(mGENER,"APP says:  STATE HAS CHANGE TO OUTDOOR\r\n");
			break;
		}
		}
	}
	if (UAlarmUp>0) Led_SetOrClr(true);
	vAlMonitor_ToNCG(sToNCG,(char *)"DMP_ALARM",uUID);
}


/** Do what it takes if Alarm Monitor noticed an alarm going DOWN
 *
 *  @param sSendAlarm sSAlarm[]: Array with all alarms data(maybe there are consequences for another alarm)
 *  @param int iPos: position in the array of the current alarm
 *
 */
void vWorkAlarmDown(sSendAlarm sSAlarm[],int iPos,sSendAlarm *sTNCG)
{
	uint_fast8_t iPointer=0;

	if (sTNCG->iTry==0)
	{
		UAlarmUp--;

		switch(sSAlarm[iPos].iOPT)
		{
		case mFUNC_PANIC:
		{

			sTNCG->cTime=(char *)Wifly_sGetTime();

			Log_print(mGENER,"APP says: Received Alarm from %i and it is %i\r\n",sSAlarm[iPos].iOPT,sSAlarm[iPos].bIsON);

			break;
		}
		case  mFUNC_ACCEL:
		{
			Log_print(mGENER,"APP says: Received Alarm anulation from mACCEL FALL and it is %i\r\n",sSAlarm[iPos].bIsON);
			break;
		}
		case  mFUNC_ACACT:
		{
			Log_print(mGENER,"APP says:Received Alarm anulation from mACCEL ACTIVITY and it is %i\r\n",sSAlarm[iPos].bIsON);
			break;
		}
		}

	}
	if (UAlarmUp==0) Led_SetOrClr(false);
	vAlMonitor_ToNCG(sTNCG,(char *)"DMP_ALARM",uUID);
}
/*-----------------------------------------------------------*/
/** Create task to run the LOG
 *
 * @void *pvParameters
 */

static void vLogTask( void *pvParameters ) {

	Log_vPrintMenu();

	for( ;; )
	{
		Log_vReadLogCmd();
		vTaskDelay( MILIS_TO_TICKS(PERIOD_TIME/10));
	}
}

//Funcs del adc

void _SendStream(int16_t iStream[],int16_t iNumElem, unsigned *iUID[5])
{
	if (iAlMonitor_GetMode()==mGOTO_INDOOR)
	{
		if (Wifly_bSendStream(iStream,iNumElem,iUID)==false)
		{
			Log_print(mGENER,"APP says: Stream has been sended by GSM\r\n");
		}
	}
	else
	{
		Log_print(mGENER,"APP says: Stream has been sended by GSM\r\n");
	}
}

/** Task to manage ADC based sensors(Temperature, accelerometer, pulsometer)
 *
 *
 * @void void
 */

static void vADCSensors( void *pvParameters )
{
	sSendAlarm xAlarmTemp;
	sReadAccel sRAccelData;
	int16_t iTemp;
	portTickType pxPrevExec=-1;
	int16_t iWaitGen = 800;
	int16_t iWaitACTE = 800;

	int16_t iPulse[1000];
	int16_t iRead=0;
	int16_t iDif = 0;
	portTickType pxFirstRead=-1;


	pxPrevExec=xTaskGetTickCount();

	for( ;; )
	{


		/*
		 *
		 * BEGIN Handle pulsometer
		 */


		if (aAlarmFuncRec[mFUNC_ACACT].bIsON==true)
		{
			if (iWaitGen == 800)
			{
				Pulse_Activate(true);
				iWaitGen = 5;
				iWaitACTE=6000;
			}

			if (pxFirstRead == -1) pxFirstRead=xTaskGetTickCount();

			iDif = (int16_t)xTaskGetTickCount()-pxFirstRead;
			iPulse[iRead]=(int16_t)ADC_Read(4);
			iRead++;

			if (iRead==1000)
			{
				pxFirstRead = -1;
				iRead=0;
				_SendStream(iPulse,1000,uUID);
				vTaskDelay( MILIS_TO_TICKS(5000));
			}
		}
		else
		{
			if (iWaitGen == 5)
			{
				Pulse_Activate(false);
				iWaitGen = 800;
				iWaitACTE = iWaitGen;
				pxFirstRead = -1;
			}
		}


		/*
		 *
		 * END Handle pulsometer
		 */


		if (xTaskGetTickCount()-pxPrevExec > iWaitACTE/portTICK_RATE_MS)
		{


			/*
			 *
			 * BEGIN Handle thermometer
			 */


			iTemp  = (int16_t)ADC_Read(3);
			if (Thermo_ChkTemp(iTemp)==true)
			{
				xAlarmTemp.iOPT = mFUNC_THERM;
				xAlarmTemp.iTry = 0;
				xAlarmTemp.bIsON = true;
				xAlarmTemp.iValue = Thermo_GetTemp(iTemp);
				xAlarmTemp.iParam = Thermo_GetZone();
				vAlMonitor_AddAlarm(&xAlarmTemp,false);
			}

			if (Thermo_ChkTMonitor(iTemp) == true)
			{
				xAlarmTemp.iOPT = mDATA_THERM;
				xAlarmTemp.iTry = 0;
				xAlarmTemp.bIsON = true;
				xAlarmTemp.iValue = Thermo_GetTemp(iTemp);
				xAlarmTemp.iParam = Thermo_GetZone();
				vAlMonitor_AddAlarm(&xAlarmTemp,false);
			}

			/*
			 *
			 * END Handle accelerometer
			 */


			/*
			 *
			 * BEGIN Handle accelerometer
			 */


			ADC_ADCAverage(&sRAccelData);
			Accel_sRAccel(&sRAccelData);


			/*
			 *
			 * END Handle accelerometer
			 */


			pxPrevExec=xTaskGetTickCount();
		}
		vTaskDelay( MILIS_TO_TICKS(iWaitGen));

	}
}

/** Init at the turn on of the clock, init the log, leds, accelerometer, panic button...
 *
 *
 * @void void
 */

void vGeneralInit(void)
{


	Log_start(mUART_LOG, BAUD_115200);

	Led_init();

	Led_SetOrClr(false);

	ButtonGPIO_vInit();

	ADC_vADCInit(1000000);

	vWifly_init(mWFPORT, BAUD_115200);

	Accel_vInit();

	Thermo_SetUp();

	Pulse_vInit();

	vAlmonitor_CreateQueue();

	Utils_ReadSerialNumber(uUID);

	return;
}


void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */

	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
